// Copyright 2020 Espressif Systems (Shanghai) PTE LTD
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
#pragma once

enum _panic_reasons {
    PANIC_RSN_NONE = 0,
    PANIC_RSN_INTWDT_CPU0,
#if SOC_CPU_NUM > 1
    PANIC_RSN_INTWDT_CPU1,
#endif
    PANIC_RSN_CACHEERR,
#if CONFIG_ESP_SYSTEM_MEMPROT_FEATURE
    PANIC_RSN_MEMPROT,
#endif
    PANIC_RSN_COUNT
} panic_reasons;
