// Copyright 2015-2018 Espressif Systems (Shanghai) PTE LTD
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at

//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#ifndef _SOC_SDIO_SLAVE_PINS_H_
#define _SOC_SDIO_SLAVE_PINS_H_

#define SDIO_SLAVE_SLOT0_IOMUX_PIN_NUM_CLK  12
#define SDIO_SLAVE_SLOT0_IOMUX_PIN_NUM_CMD  11
#define SDIO_SLAVE_SLOT0_IOMUX_PIN_NUM_D0   13
#define SDIO_SLAVE_SLOT0_IOMUX_PIN_NUM_D1   14
#define SDIO_SLAVE_SLOT0_IOMUX_PIN_NUM_D2   9
#define SDIO_SLAVE_SLOT0_IOMUX_PIN_NUM_D3   10
#define SDIO_SLAVE_SLOT0_FUNC               0

#define SDIO_SLAVE_SLOT1_IOMUX_PIN_NUM_CLK  36
#define SDIO_SLAVE_SLOT1_IOMUX_PIN_NUM_CMD  35
#define SDIO_SLAVE_SLOT1_IOMUX_PIN_NUM_D0   37
#define SDIO_SLAVE_SLOT1_IOMUX_PIN_NUM_D1   38
#define SDIO_SLAVE_SLOT1_IOMUX_PIN_NUM_D2   33
#define SDIO_SLAVE_SLOT1_IOMUX_PIN_NUM_D3   34
#define SDIO_SLAVE_SLOT1_FUNC               3

#endif /* _SOC_SDIO_SLAVE_PINS_H_ */
