// Copyright 2020 Espressif Systems (Shanghai) PTE LTD
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at

//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#ifndef _SOC_SPI_PINS_H_
#define _SOC_SPI_PINS_H_

#define SPI_FUNC_NUM            0
#define SPI_IOMUX_PIN_NUM_HD    12
#define SPI_IOMUX_PIN_NUM_CS    14
#define SPI_IOMUX_PIN_NUM_MOSI  16
#define SPI_IOMUX_PIN_NUM_CLK   15
#define SPI_IOMUX_PIN_NUM_MISO  17
#define SPI_IOMUX_PIN_NUM_WP    13

#define SPI2_FUNC_NUM           2
#define SPI2_IOMUX_PIN_NUM_MISO 2
#define SPI2_IOMUX_PIN_NUM_HD   4
#define SPI2_IOMUX_PIN_NUM_WP   5
#define SPI2_IOMUX_PIN_NUM_CLK  6
#define SPI2_IOMUX_PIN_NUM_MOSI 7
#define SPI2_IOMUX_PIN_NUM_CS   10

#endif
